
#include <adc.h>
#include "micro_frc.h"
#include "adc.h"


// Define Message Buffer Length for ECAN1/ECAN2
#define  MAX_CHNUM	 			12		// Highest Analog input number in Channel Scan
#define  SAMP_BUFF_SIZE	 		8		// Size of the input buffer per analog input
#define  NUM_CHS2SCAN			7		// Number of channels enabled for channel scan

// Number of locations for ADC buffer = 14 (AN0 to AN13) x 8 = 112 words
// Align the buffer to 128 words or 256 bytes. This is needed for peripheral indirect mode
unsigned int BufferA[MAX_CHNUM][SAMP_BUFF_SIZE] __attribute__((space(dma),aligned(256)));
unsigned int BufferB[MAX_CHNUM][SAMP_BUFF_SIZE] __attribute__((space(dma),aligned(256)));

void initDma0(void);

/*
 *   This function initializes the dsPIC ADC module:
 *      - 12-bit operation
 *      - AVDD and AGND as reference
 *      - use channel 0, 1, 3, 4, 5, 6, 7
 *      - Use DMA
 */
void adc_open()
{
	AD1CON1bits.FORM   = 0b10;		// Data Output Format: Fractional
	AD1CON1bits.SSRC   = 0b111;	// Sample Clock Source: internal timer
	AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
	AD1CON1bits.AD12B  = 1;		// 12-bit ADC operation
	

	AD1CON2bits.CSCNA = 1;		// Scan Input Selections for CH0+ during Sample A bit
	AD1CON2bits.CHPS  = 0;		// Converts CH0
	AD1CON2bits.VCFG  = 0b000;  // use AVDD and AGND

	AD1CON3bits.ADRC = 0;		// ADC Clock is derived from Systems Clock
	AD1CON3bits.ADCS = 63;		// ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*64 = 1.6us (625Khz)
								// ADC Conversion Time for 10-bit Tc=12*Tab = 19.2us	

	AD1CON1bits.ADDMABM = 0; 	// DMA buffers are built in scatter/gather mode
	AD1CON2bits.SMPI    = (NUM_CHS2SCAN-1);	// 4 ADC Channel is scanned
	AD1CON4bits.DMABL   = 3;	// Each buffer contains 8 words

	//AD1CSSH/AD1CSSL: A/D Input Scan Selection Register
	AD1CSSH = 0x0000;			
	AD1CSSL = 0x0000;
	AD1CSSLbits.CSS0 = 1;			// Enable AN0 for channel scan
	AD1CSSLbits.CSS1 = 1;
	AD1CSSLbits.CSS3 = 1;
	AD1CSSLbits.CSS4 = 1;
	AD1CSSLbits.CSS5 = 1;
	AD1CSSLbits.CSS6 = 1;
	AD1CSSLbits.CSS7 = 1;

 
 	//AD1PCFGH/AD1PCFGL: Port Configuration Register
	AD1PCFGL=0xFFFF;
	AD1PCFGH=0xFFFF;
	AD1PCFGLbits.PCFG0 = 0;		// AN0 as Analog Input
	AD1PCFGLbits.PCFG1 = 0;
 	AD1PCFGLbits.PCFG3 = 0;
	AD1PCFGLbits.PCFG4 = 0;
	AD1PCFGLbits.PCFG5 = 0;
	AD1PCFGLbits.PCFG6 = 0;
	AD1PCFGLbits.PCFG7 = 0;
	
	IFS0bits.AD1IF   = 0;		// Clear the A/D interrupt flag bit
	IEC0bits.AD1IE   = 0;		// Do Not Enable A/D interrupt 
	
	initDma0();
	
	adc_start();
}


void adc_start()
{
	AD1CON1bits.ADON = 1;		// Turn on the A/D converter
}


void adc_stop()
{
	AD1CON1bits.ADON = 0;		// Turn off the A/D converter	
}		


/*
 *    Enable dsPIC DMA module. ADC results will be pushed to
 *    BufferA and BufferB according to the ping-pong mechanism.
 *    
 */
void initDma0(void)
{
	DMA0CONbits.AMODE = 2;			// Configure DMA for Peripheral indirect mode
	DMA0CONbits.MODE  = 2;			// Configure DMA for Continuous Ping-Pong mode
	DMA0PAD=(int)&ADC1BUF0;
	DMA0CNT = (SAMP_BUFF_SIZE*NUM_CHS2SCAN)-1;					
	DMA0REQ = 13;					// Select ADC1 as DMA Request source

	DMA0STA = __builtin_dmaoffset(BufferA);		
	DMA0STB = __builtin_dmaoffset(BufferB);

	IFS0bits.DMA0IF = 0;			//Clear the DMA interrupt flag bit
    IEC0bits.DMA0IE = 1;			//Set the DMA interrupt enable bit

	DMA0CONbits.CHEN=1;				// Enable DMA
}


unsigned int middle(unsigned int x, unsigned int y, unsigned int z)
{
	if (x > y)
	{
		if (z > x)
			return x;
		else if (z > y)
			return z;
		else
			return y;	
	} 
	else // x < y 
	{
		if (z < x)
			return x;
		else if (z < y) // & x < z
			return z;
		else
			return y;		
	}
}


void ProcessADCSamples(unsigned int * AdcBuffer)
{
	int i;
	unsigned long s = 0;
	for (i = 0; i < 8; i++)
	{
		s += AdcBuffer[i];
	}
	s /= 8;
	AdcBuffer[0] = (unsigned int)s;
	//printf (" %u ", (int) (s>>4));
}


unsigned int DmaBuffer = 0;

void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{
	if(DmaBuffer == 0)
	{
		ProcessADCSamples(&BufferA[0][0]); 
		ProcessADCSamples(&BufferA[1][0]);
		//ProcessADCSamples(&BufferA[2][0]);    // we don't use AN2
		ProcessADCSamples(&BufferA[3][0]);
		ProcessADCSamples(&BufferA[4][0]);
		ProcessADCSamples(&BufferA[5][0]);
		ProcessADCSamples(&BufferA[6][0]);
		ProcessADCSamples(&BufferA[7][0]);
	}
	else
	{
		ProcessADCSamples(&BufferB[0][0]);
		ProcessADCSamples(&BufferB[1][0]);
		//ProcessADCSamples(&BufferB[2][0]);    // we don't use AN2
		ProcessADCSamples(&BufferB[3][0]);
		ProcessADCSamples(&BufferB[4][0]);
		ProcessADCSamples(&BufferB[5][0]);
		ProcessADCSamples(&BufferB[6][0]);
		ProcessADCSamples(&BufferB[7][0]);
		adc_stop();
	}

	DmaBuffer ^= 1;

	IFS0bits.DMA0IF = 0;		// Clear the DMA0 Interrupt Flag
	
}


unsigned int adc_get_channel(int i)
{
	return (BufferB[i][1]/2 + BufferA[i][1]/2);
}

