
// ahrs.c
#include <stdio.h>
#include "math.h"
#include "ahrs.h"

// define printdebugstuff to print verbose output to stdout
//#define printdebugstuff


/*
Extended Kalman filter application

Reference - Kingston and Beard Paper "Real-Time Attitude and Position 
Estimation for Small UAVs Using Low-Cost Sensors"

Estimates attitude from 3 accelerometers, 3 gyros, and a heading input (compass or gps)

n = 7 states in X vector as q0,q1,q2,q3,pbias,qbias,rbias
r = 3 outputs 

matrix format is
[ 0,0  0,1  0,2  0,3]
[ 1,0  1,1  1,2	 1,3]
[ 2,0  2,1  2,2  2,3]
[ 3,0  3,1  3,2  3,3]

*/


// Kalman weighting matrices
// process noise covariance - how much we trust the acceleromters relative to the gyros.
m_t Q = {												
	{ 0.0001,		0.,			0.,			0.,			0.,		0.,		0.		},	// q0	q_angle
	{ 0.,			0.0001,		0.,			0.,			0.,		0.,		0.		},	// q1	
	{ 0.,			0.,			0.0001,		0.,			0.,		0.,		0.		},	// q2
	{ 0.,			0.,			0.,			0.0001,		0.,		0.,		0.		},	// q3
	{ 0.,			0.,			0.,			0.,			0.03,	0.,		0.		},	// p bias	q_gyro
	{ 0.,			0.,			0.,			0.,			0.,		0.03,	0.		},	// q bias
	{ 0.,			0.,			0.,			0.,			0.,		0.,		0.03		}	// r bias
	
};

// measurement noise covariance (error) we expect 0.3 rad jitter , smaller - the faster we converge from gyro towards accelerometer angle?
// about 2 deg = .035 rad
m_t R = {										
	{ 	0.1,		0.000,		0.000 	},		// r_angle 
	{ 	0.000,		0.1,		0.000 	},		
	{ 	0.000,		0.000,		0.1  	}			
};

/*
 * Covariance matrix - initialized in ahrs_init()
 */
m_t P;


/*
 * State vector first 4 values are quaterion of attitude estimate q0,q1,q2,q3 ie. Level, facing north is { 1.0, 0.0, 0.0, 0.0 }
 * last 3 values are 3 axis gyro bias values in radians p_bias, q_bias, r_bias for roll, pitch and yaw
 * Initialized in ahrs_init() .
 */
v_t X;


/**
 *  ahrs_init() initial state vector with the quaterion attitude 
 * calculated from the steady state accelerometer values and the
 * bias from the initial gyro measurements.
 * Also initialize the P matrix based on some assumptions of the
 * initial sensor measurements.
 */
void ahrs_init(
	f_t			ax,			// x axis acceleration
	f_t			ay,			// y 
	f_t			az,			// z
	f_t			p_bias,		// initial steady state roll gyro measurement
	f_t			q_bias,		// pitch gyro
	f_t			r_bias,		// yaw gyro
	f_t			heading		// initial compass or gps heading
)
{
	v_t			euler_angles;


// get the euler angles from initial accelerometer readings
		accel2euler(
		euler_angles,
		ax,
		ay,
		az,
		heading
	);

	v_print("initial euler_angles (radians)",euler_angles,3);

	euler2quat(			// initialize the X vector with quaternions representing initial attitude - derived from accelerometers
		X,
		euler_angles[0],
		euler_angles[1],
		euler_angles[2]
	);

// gyro bias
// initialize state vector with gyro biases with current (assumed static) gyro reading
	X[4]= p_bias;
	X[5]= q_bias;
	X[6]= r_bias;

	v_print("initial state vector X",X,7);

	v_normq( X, X );			// normalize the quaterion

	v_print("normalized state vector X",X,7);

// initialize P matrix - error covariance, assume diagonal
m_zero(P,7,7);  //zero all

// manually set diagonals
// top 4 diagonals are position confidence - big number is low confidence
P[0][0]=.1;		// q0 confidence
P[1][1]=.1;		// q1
P[2][2]=.1;		// q2
P[3][3]=.1;		// q3

// bottom 3 diagonals are bias confidence - small number is hi confidence
//P[4][4]=.000001;
//P[5][5]=.000001;
//P[6][6]=.000001;
P[4][4]=.0001;
P[5][5]=.0001;
P[6][6]=.0001;

		m_print("initial P",P,7,7);


}

/**
 *  ahrs_step() takes the values from the IMU and produces the
 *  new estimated attitude.
 							P_dot = dt(AP + PA' + Q)
							P = P + P_dot
							q = q + dt( omega(w-b) )q
							calls kalman update
				
 */
void ahrs_step(
	v_t			angles_out,	// return euler angles for attitude phi,theta,psi for roll, pitch, and yaw
	f_t			dt,
	f_t			p, 		// measured rates
	f_t			q,
	f_t			r,
	f_t			ax, 	// measured accels
	f_t			ay,
	f_t			az,
	f_t			heading // gps or compass heading
)
{
	m_t			A; 		// 7x7
	m_t			AT;		// 7x7
	m_t			P_dot;	// 7x7
	m_t			temp;	// 7x7
	v_t			qhat;	// 4
	v_t			Xm;		// 3
	v_t			Xe;		//3

#ifdef printdebugstuff
		printf( "start of ahrs step\n\r");
		printf( "p=%lf q=%lf r=%lf\n\r", p,q,r );		
#endif


	// Construct matrix A (7x7) from the 3 gyro measurements and the current state vector
	// current state vector is quaterion of position and bias estimates
	build_A( A, p, q, r ); 

#ifdef printdebugstuff
		v_print("X",X,7); 
		m_print("A",A,7,7);
#endif

// calculate P_dot = AP + PA' + Q

	m_transpose( AT, A, 7, 7 );  		// AT = A' 

#ifdef printdebugstuff
		m_print("AT",AT,7,7);
#endif


	m_mult( temp, P, AT, 7, 7, 7 ); 	// temp = PA'

#ifdef printdebugstuff
		m_print("PA'",temp,7,7);
#endif

	m_mult(P_dot, A, P, 7, 7, 7 ); 		// P_dot = AP

#ifdef printdebugstuff
		m_print("AP",P_dot,7,7);
#endif

	m_add(P_dot, P_dot, temp, 7, 7 ); 	// P_dot = AP + PA'	

#ifdef printdebugstuff
		m_print("AP +PA'",P_dot,7,7);
#endif


	m_add(P_dot, P_dot, Q, 7, 7 ); 		// P_dot = AP + PA' + Q

#ifdef printdebugstuff
		m_print("AP +PA' + q",P_dot,7,7);
#endif


	m_scale(P_dot, P_dot, dt, 7, 7);	//	P_dot = dt(AP + PA' + Q)

#ifdef printdebugstuff
		m_print("DT(AP +PA' + q)",P_dot,7,7);
#endif

	m_add( P, P, P_dot, 7,7 );			// P = P + P_dot

#ifdef printdebugstuff
		m_print("NEW P",P,7,7);		
#endif

// still have omega(measured gyro rate - bias) in top left 4x4 of A
// X has q0-q3 in first 4 numbers

#ifdef printdebugstuff
		v_print("X",X,7);
		m_print("A_11",A,4,4);		
#endif

	mv_mult(qhat,A,X,4,4);		// qhat = omega(w-b)q


#ifdef printdebugstuff
		v_print("qhat",qhat,4); 
#endif

	v_scale (qhat,qhat,dt,4);	// times dt

#ifdef printdebugstuff
		v_print("qhat scaled",qhat,4); 
#endif

	v_add(X,X,qhat,4);			// x = x + qhat		UPDATE X with time upedate

#ifdef printdebugstuff
		v_print("X",X,7);		
#endif

	v_normq( X, X );			// normalize

#ifdef printdebugstuff
		v_print("updated X normalized",X,7); 
#endif

	// Compute the euler angles of the measured attitude from accelerometer measurements
	accel2euler(Xm, ax,	ay,	az,	heading	);				

#ifdef printdebugstuff
	v_print("Xm",Xm,3);
#endif

	 // Compute the euler angles of the estimated from q0-q3
	 // Xe = quat2euler( X ) = (estimated state)
	 
	quat2euler( Xe, X );									

#ifdef printdebugstuff
	v_print("Xe",Xe,3);
#endif

	// kalman_update(Xe, Xm )  other variables are now globals
	kalman_update( Xe, Xm );
	
	// extract the euler angles from the quaterion in X
	quat2euler( angles_out, X );

#ifdef printdebugstuff
	v_print("angles_out",angles_out,3);
#endif

/*
//printf( "%1f %1f %1f %1f %1f %1f %1f %1f %1f %1f %1f %1f\n", 
printf( "%8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n", 
			angles_out[0]*57.2958,			// phi roll
			angles_out[1]*57.2958,			// theta pitch
			angles_out[2]*57.2958,			// psi yaw
			Xm[0]*57.2958,					// accel only roll
			Xm[1]*57.2958,					// pitch
			Xm[2]*57.2958,					// yaw
			p*57.2958,						// gyro measurements
			q*57.2958,
			r*57.2958,
			X[4]*57.2958,					// gyro bias
			X[5]*57.2958,
			X[6]*57.2958
		   );
*/
/*
printf( "%1f %1f %1f %1f %1f\n", 
			angles_out[1]*57.2958,			// phi pitch
			Xm[1]*57.2958,					// accel only pitch
			q*57.2958,						// gyro measurements
			X[5]*57.2958,					// gyro bias
			(q-X[5])*57.2958				// q - bias
		   );
*/


} // end ahrs_step


// Kalman filter update

void kalman_update(
//	m_t			P, 		// [7,7] covariance matrix
//	m_t			R,		// [3,3] measurement weights
//	m_t			C,		// [3,7] 
//	v_t			X,		// [7,1] estimated state as a quaterion
//const v_t	Xe,			// [3,1] euler angles of the estimated state
//const v_t	Xm			// [3,1] measured euler angles
	v_t	Xe,				// [3,1] euler angles of the estimated state
	v_t	Xm				// [3,1] measured euler angles
)
{
	m_t			C;		// 3x7
//	m_t			CT;		// 7x3
	m_t			T1;		// 7x3, 3x3
	m_t			T2;		// 7x3
	m_t			E;		// 3x3
	m_t			K;		// 7x3
	v_t			err;	// 3
	v_t			temp;	// 7

#ifdef printdebugstuff
		printf( "start of kalman update\n\r");
		v_print("X",X,7);
		v_print("Xe",Xe,3);
		v_print("Xm",Xm,3);
#endif

	build_C(C, X);	// build the C matrix

#ifdef printdebugstuff
		m_print("C",C,3,7);
		m_print("P",P,7,7);
#endif	

	//  T1[7,3] = (transpose)C[3,7]
	m_transpose( T1, C, 3, 7 );

#ifdef printdebugstuff
		m_print("C'",T1,7,3);
#endif

	// T2[7,3] = P[7,7] * C'[7,3]			PC' = T2 will be used later also
	m_mult(T2,P,T1,7,7,3);
	
#ifdef printdebugstuff
		m_print("PC'",T2,7,3);
#endif

	
	// T1[3,3] = C[3,7] * T2[7,3] = CPC'
	m_mult( T1, C, T2, 3, 7, 3 ); // 3 height of C, 7 width of C = height of T2, last 3 is width of T2


#ifdef printdebugstuff
		m_print("CPC'",T1,3,3);
		m_print("R'",R,3,3);
#endif

	//E[3,3] = T1[3,3] + R[3,3] = CPC' + R
	m_add( E, T1, R, 3, 3 );						// E = CPC' +R


#ifdef printdebugstuff
		m_print("E = CPC'+ R",E,3,3);	
#endif


	// T1[3,3] = E[3,3](-1)
	m33_inv( T1, E );

#ifdef printdebugstuff
		m_print("E(-1)",T1,3,3);	
#endif					

	// still have T2 = PC' from above
	// K[7,3] = T2[7,3] * T1[3,3] = PC' * E(-1)
	m_mult( K, T2, T1, 7, 3, 3 );					// K = PC'E(-1) done with K

#ifdef printdebugstuff
		m_print("Kalman gain K",K,7,3);	
#endif

	// update the state vector
	// err[3]=Xm[3]-Xe[3]
	v_sub( err, Xm, Xe, 3 );						// these are euler angles, error = measured angles - estimated angles


#ifdef printdebugstuff
		v_print("err",err,3);	
#endif


// may have to correct vector position here for Xm-Xe

	// temp[7] = K[7,3] * err[3] = K(Xm-Xe)
	mv_mult( temp, K, err, 7, 3 );					// 7,3 ???????????

#ifdef printdebugstuff
		v_print("temp",temp,7);	
#endif


	v_add( X, X, temp, 7 );							// X = X + K(Xm-Xe)

#ifdef printdebugstuff
		v_print("X",X,7);
#endif

	v_normq( X, X );			// normalize

//*************************************************************
	//v_print("X normalized",X,7);


	/* P[7,7] = P[7,7] - K[7,3] C[3,7] P[7,7] */
	// T1[7,7] = K[7,3] C[3,7]
	m_mult( T1, K, C, 7, 3, 7 );

#ifdef printdebugstuff
		m_print("KC",T1,7,7);	
#endif


	// T2[7,7] = T1[7,7] * P[7,7] = KCP
	m_mult( T2, T1, P, 7, 7, 7 );

#ifdef printdebugstuff
		m_print("KCP",T2,7,7);	
#endif


	// P[7,7] = P[7,7] - T2[7,7] = 
	m_sub( P, P, T2, 7, 7 );						// P = P - KCP

#ifdef printdebugstuff
		m_print("new P",P,7,7);	
#endif

} // end kalman_update



// correct angles to ECEF reference frame based on gps acceleration
// this will update phi and theta angle[1] and [2]
void body_to_ecef(
				v_t 		e_angles,	// 3 euler angles input, corrected angles output phi,theta,psi	
				f_t			ax, 	// measured accels
				f_t			ay,
				f_t			az,
				f_t			agpsx,		// 3 axis acceleration from gps calcs
				f_t			agpsy,
				f_t			agpsz
				)		
{
//f_t		a_x;		// reference acceleration vector
//f_t		a_y;
//f_t		a_z;

f_t		g = 9.81;		// gravity

f_t		rx;			
f_t		ry;
f_t		rz;
f_t		sigma_theta;
f_t		r_theta;
f_t		sigma_phi;

//a_x = -sin(e_angles[2]) * g;
//a_y =  cos(e_angles[2]) * sin(e_angles[1]) * g;
//a_z =  cos(e_angles[2]) * cos(e_angles[1]) * g;

rx = -( cos(e_angles[2]) * agpsx + sin(e_angles[2]) * agpsy);
ry = -(-sin(e_angles[2]) * agpsx + cos(e_angles[2]) * agpsy);
rz = g - agpsz;

sigma_theta = (rx * ax + rz * sqrt(rx * rx + rz * rz - ax * ax)) / (rx * rx + rz * rz);

//theta
e_angles[1] = atan2(sigma_theta * rx - ax, sigma_theta*rz); 

r_theta = rx * sin(e_angles[1]) + rz * cos(e_angles[1]);


sigma_phi = (ry * ay + r_theta * sqrt(ry * ry + r_theta * r_theta - ay * ay)) / (ry * ry + r_theta * r_theta);

//phi
e_angles[0] = atan2(-sigma_phi * ry + ay, sigma_phi * r_theta);  //may be reversed


}




/**
 *  Vector helpers related to the AHRS computation
 */
f_t sq(f_t d)
{
	return d * d;
}

/*
 * This will normalize a quaternion vector q
 * q/norm(q)
 * q(4,1)
 */
void v_normq(
	v_t			q_out,
	const v_t		q
)
{
	v_scale(
		q_out,
		q,
		1.0 / sqrt( sq(q[0]) + sq(q[1]) + sq(q[2]) + sq(q[3]) ),
		4
	);
}


/*
 * This will convert from quaternions to euler angles
 * q(4,1) -> euler[phi;theta;psi] (rad)
 */
void quat2euler(
	v_t			euler,
	const v_t		q
)
{
	f_t			q0 = q[0];
	f_t			q1 = q[1];
	f_t			q2 = q[2];
	f_t			q3 = q[3];

//	f_t			q02 = q0*q0;
	f_t			q12 = q1*q1;
	f_t			q22 = q2*q2;
	f_t			q32 = q3*q3;

	/* phi */
	euler[0] = atan2( 2.0 * (q2*q3 + q0*q1), 1.0 - 2.0 * (q12 + q22) );

	/* theta */
	euler[1] = -asin( 2.0 * (q1*q3 - q0*q2) );

	/* psi */
    euler[2] = atan2( 2.0 * (q1*q2 + q0*q3), 1.0 - 2.0 * (q22 + q32) );
}
	
/*
 * This will convert from euler angles to quaternion vector
 * phi, theta, psi -> q(4,1)
 * euler angles in radians
 */
void euler2quat(
	v_t			q,
	f_t			phi,
	f_t			theta,
	f_t			psi
)
{
	f_t			sphi	= sin( 0.5 * phi );
	f_t			stheta	= sin( 0.5 * theta );
	f_t			spsi	= sin( 0.5 * psi );

	f_t			cphi	= cos( 0.5 * phi );
	f_t			ctheta	= cos( 0.5 * theta );
	f_t			cpsi	= cos( 0.5 * psi );

	q[0] =  cphi*ctheta*cpsi + sphi*stheta*spsi;
	q[1] = 	sphi*ctheta*cpsi - cphi*stheta*spsi; //signs ??? MR
	q[2] =  cphi*stheta*cpsi + sphi*ctheta*spsi;
	q[3] =  cphi*ctheta*spsi - sphi*stheta*spsi;
}



/*
 * This will convert from accelerometer and compass readings into
 * [phi, theta, psi] (in radians).  The accelerometer readings are
 * assumed to be in m/s^2, but the actual units don't matter.
 *
 * Requires 13250 instructions (3310 microseconds)
 */
void accel2euler(
	v_t			euler,
	f_t			ax,
	f_t			ay,
	f_t			az,
	f_t			compass
)
{
	f_t			g = sqrt( ax*ax + ay*ay + az*az );

	euler[0] = atan2( ay, -az );		// Roll 
    euler[1] = asin( ax / -g );			// Pitch
    euler[2] = compass;					// Yaw
}


void build_C(m_t W, v_t q)	//build the C matrix (W here) from the state vector X (q here) 
{
// c is a 7x3
f_t a,b,c,g,s; // to simplify intermediate calculations

// top row is roll dphi/dq part

	a = 2. * (q[0] * q[1] + q[2] * q[3]);
	c = 1. - 2.*(q[1] * q[1] + q[2] * q[2]);
	s = (2. * c) / (a * a + c * c);

W[0][0]=	s * q[1];
W[0][1]=	s * (q[0] + 2. * q[1] * a);
W[0][2]=	s * (q[3] + 2. * q[2] * a);
W[0][3]=	s * q[2];
W[0][4]= 0.;
W[0][5]= 0.;
W[0][6]= 0.;

// second row is pitch dtheta/dq part

g = -2. * (q[1] * q[3] - q[0] * q[2]);
s = 2. / sqrt(1. - g * g);

W[1][0]=	 s * q[2];
W[1][1]=	-s * q[3];
W[1][2]=	 s * q[0];
W[1][3]=	-s * q[1];
W[1][4]= 0.;
W[1][5]= 0.;
W[1][6]= 0.;

// third row is yaw dpsi/dq part

a = 2. * (q[1] * q[2] + q[0] * q[3]);
b = 1. - 2. * (q[2] * q[2] + q[3] * q[3]);
s = (2. * b) / (a * a + b * b);


W[2][0]=	s * q[3];
W[2][1]=	s * q[2];
W[2][2]=	s * (q[1] + 2. * q[2] * a);
W[2][3]=	s * (q[0] + 2. * q[3] * a);
W[2][4]= 0.;
W[2][5]= 0.;
W[2][6]= 0.;

}



/*
 * This will construct the quaternion omega matrix
 * W(7,7)
 * p, q, r (rad/sec)
 */
void build_A(
	m_t			W,
	f_t			p,
	f_t			q,
	f_t			r
)
{   f_t		wmb1 = p - X[4];
	f_t		wmb2 = q - X[5];
	f_t		wmb3 = r - X[6];

	//A11 (one one) part (4x4) is omega(measured rates - estimated bias)
	W[0][0] = 0.0;
	W[0][1] =  -wmb1/2.0; // signs ok some reversed to minimize negates
	W[0][2] =  -wmb2/2.0;
	W[0][3] =  -wmb3/2.0;

	W[1][0] =   wmb1/2.0;
	W[1][1] =   0.0;
	W[1][2] =   wmb3/2.0;
	W[1][3] =  -wmb2/2.0;

	W[2][0] =   wmb2/2.0;
	W[2][1] =  -wmb3/2.0;
	W[2][2] =   0.0;
	W[2][3] =   wmb1/2.0;

	W[3][0] =   wmb3/2.0;
	W[3][1] =   wmb2/2.0;
	W[3][2] =  -wmb1/2.0;
	W[3][3] =   0.0;

	// this is the A12 part, a function of the quaterion position

	W[0][4] =  X[1]/2.0;
	W[0][5] =  X[2]/2.0;
	W[0][6] =  X[3]/2.0;

	W[1][4] = -X[0]/2.0;
	W[1][5] =  X[3]/2.0;
	W[1][6] = -X[2]/2.0;

	W[2][4] = -X[3]/2.0;
	W[2][5] = -X[0]/2.0;
	W[2][6] =  X[1]/2.0;	

	W[3][4] =  X[2]/2.0;
	W[3][5] = -X[1]/2.0;
	W[3][6] = -X[0]/2.0;

	// bottom 3 rows should be zero

	v_zero(W[4],7);
	v_zero(W[5],7);
	v_zero(W[6],7);
}

// vector functions!!!!!!!!!!!!!!!!!!!!!!!!!!!

 void v_zero(v_t c, iter_t n)
{
do {
		--n;
		c[n] = 0.;
	} while( n > 0 );
}


 void v_add(
	v_t 			c,
	const v_t 		a,
	const v_t 		b,
	iter_t			n
)
{
	do {
		--n;
		c[n] = a[n] + b[n];
	} while( n > 0 );
}


 void v_sub(					//c=a-b
	v_t 			c,
	const v_t 		a,
	const v_t 		b,
	iter_t			n
)
{
	do {
		--n;
		c[n] = a[n] - b[n];
	} while( n > 0 );
}


 void v_scale(
	v_t			c,
	const v_t	a,
	f_t			s,
	iter_t		n
)
{
	do {
		--n;
		c[n] = a[n] * s;
	} while( n > 0 );
}

// Matrix operations !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 void m_zero(m_t c, iter_t m, iter_t n)
{
do {
		--n;
	v_zero( c[n], m );
	} while( n > 0 );
}

/*
 * Adds two matrices: c = a + b
 *
 * Safe to call with c == a and c == b
 * Requires 2300 instructions (580 useconds)
 */
 void m_add(
	m_t 		c,
	m_t 		a,
	m_t			b,
	iter_t		m,
	iter_t		n
)
{
	do {
		n--;
		v_add( c[n], a[n], b[n], m );
	} while( n > 0 );
}

 void m_sub(
	m_t 		c,
	m_t 		a,
	m_t			b,
	iter_t		m,
	iter_t		n
)
{
	do {
		n--;
		v_sub( c[n], a[n], b[n], m );		//c=a-b
	} while( n > 0 );
}

 void m_scale(
	m_t			c,
	m_t			a,
	f_t			s,
	iter_t		m,
	iter_t		n
)
{
	do {
		n--;
		v_scale( c[n], a[n], s, m );
	} while( n > 0 );
}

/**
 *  vc = ma * vb
 * Unsafe to call with vc == vb
 */
 void mv_mult(
	v_t			vc,
	m_t			ma,
	const v_t	vb,
	iter_t		m,
	iter_t		n
)
{
	iter_t		i;
	iter_t		j;

	for( j=0 ; j<m ; ++j )
	{
		f_t			sum = (f_t) 0;			// ??????????????????
		const f_t *		row = &ma[j][0];

		for( i=0 ; i<n ; ++i )
			sum += row[i] * vb[i];

		vc[j] = sum;
	}
}


/**
 *  vc = va * mb
 * Unsafe to call with vc == va
 */
 void vm_mult(
	v_t			vc,
	v_t			va,
	m_t			mb,
	iter_t		m,
	iter_t		n
)
{
	iter_t		i;
	iter_t		j;

	for( i=0 ; i<n ; ++i )
	{
		f_t			sum = (f_t) 0;

		for( j=0 ; j<m ; j++ )
			sum += va[j] * mb[j][i];

		vc[j] = sum;
	}
}

/**
 *  c = a * b
 * Unsafe for c == a or c == b.
 * Optimal form runs in 1/2 the time of the simple form due to
 * loop constant hoisting and others.
 */
 void m_mult(
	m_t			c,
	m_t			a,
	m_t			b,
	iter_t		m,
	iter_t		n,
	iter_t		p
)
{
	iter_t		i;
	iter_t		j;
	iter_t		k;
/*
// start simple
	m_zero( c );

	for( i=0 ; i<m ; ++i )
		for( j=0 ; j<p ; ++j )
			for( k=0 ; k<n ; ++k )
				c[i][j] += a[i][k] * b[k][j];
//end simple
*/
//start not simple
	for( i=0 ; i<m ; ++i )
	{
		for( j=0 ; j<p ; ++j )
		{
			f_t		sum = (f_t) 0;
			const f_t *	A = &a[i][0];

			for( k=0 ; k<n ; ++k )
				sum += A[k] * b[k][j];
			c[i][j] = sum;
		}
	}
// end not simple

}


//transpose is safe to call with c == a
 void m_transpose(
	m_t			c,
	m_t			a,
	iter_t		m,
	iter_t		n
)
{
	iter_t		i;
	iter_t		j;

	for( i=0 ; i<m ; ++i )	//i is m
	{
		for( j=i ; j<n ; j++ )	//j is n
		{
			f_t			u = a[i][j];
			f_t			l = a[j][i];

			c[j][i] = u;
			c[i][j] = l;
		}
	}
}

//Produce the determinant of a 3x3 matrix
 f_t m33_det(
	m_t			m
)
{
	return	m[0][0] * m[1][1] * m[2][2]
	- 	m[0][0] * m[1][2] * m[2][1]
	-	m[0][1] * m[1][0] * m[2][2]
	+	m[0][1] * m[1][2] * m[2][0]
	+	m[0][2] * m[1][0] * m[2][1]
	-	m[0][2] * m[1][1] * m[2][0]
	;
}

/**
 *  m33_inv will invert a 3x3 matrix.
 *
 * Unsafe to call with c == a
 *
 * Details were copied from:
 *
 *	http://mathworld.wolfram.com/MatrixInverse.html
 *
 * Requires 16478 instructions (4120 useconds)
 */
 void m33_inv(
	m_t		D,
	m_t		m
)
{
	D[0][0] = m[1][1] * m[2][2] - m[1][2] * m[2][1];
	D[0][1] = m[0][2] * m[2][1] - m[0][1] * m[2][2];
	D[0][2] = m[0][1] * m[1][2] - m[0][2] * m[1][1];

	D[1][0] = m[1][2] * m[2][0] - m[1][0] * m[2][2];
	D[1][1] = m[0][0] * m[2][2] - m[0][2] * m[2][0];
	D[1][2] = m[0][2] * m[1][0] - m[0][0] * m[1][2];

	D[2][0] = m[1][0] * m[2][1] - m[1][1] * m[2][0];
	D[2][1] = m[0][1] * m[2][0] - m[0][0] * m[2][1];
	D[2][2] = m[0][0] * m[1][1] - m[0][1] * m[1][0];

	m_scale( D, D, 1.0 / m33_det(m), 3, 3 );
}

 void m_print(
	const char *		label,
	m_t			a,
	iter_t		m,
	iter_t		n
)
{
	iter_t		i;
	iter_t		j;

	printf( "%s =\n", label );
	for( i=0 ; i<m ; i++ )
	{
		printf( "[" );
		for( j=0 ; j<n ; j++ )
			printf( " %lf", a[i][j] );
		printf( "]\n" );
	}
	printf( "\r" );
}


void v_print(
	const char *		label,
	v_t			a,
	iter_t		m
	
)
{
	iter_t		i;
	
	printf( "%s =\n", label );
	for( i=0 ; i<m ; i++ )
	printf( " %lf", a[i] );

	printf( "\n\r" );
}
